/**
 * apeman task to rewrite hosts file.
 * @memberof module:apeman-task-hosts/lib
 * @function define
 * @param {object} hosts - Values to set.
 * @param {object} [options] - Optional settings.
 * @returns {function} - Defined task function.
 */

'use strict'

const co = require('co')
const hostile = require('hostile')
const clc = require('cli-color')
const objnest = require('objnest')

/** @lends define */
function define (hosts, options = {}) {

  let removing = !!options.remove
  let listing = !!options.list

  if (removing && listing) {
    throw new Error(`Invalid options: ${JSON.parse(options)}`)
  }

  let { bold } = clc.black

  /**
   * Defined task.
   * @function task
   * @param {object} context - Apeman task context.
   * @returns {Promise}
   */
  function task (context) {
    let { logger } = context

    if (typeof(hosts) === 'function') {
      hosts = hosts()
    }

    return co(function * () {

      if (listing) {
        let lines = yield new Promise((resolve, reject) =>
          hostile.get(false, (err, lines) => err ? reject(err) : resolve(lines))
        )
        logger.debug(`--- Host settings ---`)
        lines.forEach((item) => {
          if (item.length > 1) {
            logger.debug(item[ 0 ], bold(item[ 1 ]))
          } else {
            logger.debug(item)
          }
        })
        logger.debug('')
        return
      }
      for (let hostname of Object.keys(hosts)) {
        let ip = hosts[ hostname ];
        if (removing) {
          logger.debug(`Remove host:\t${ip} ${bold(hostname)}`)
          yield new Promise((resolve, reject) =>
            hostile.remove(ip, hostname, (err) => err ? reject(err) : resolve())
          )
        } else {
          logger.debug(`Add host:\t${ip} ${bold(hostname)}`)
          yield new Promise((resolve, reject) =>
            hostile.set(ip, hostname, (err) => err ? reject(err) : resolve())
          )
        }
      }
    })
  }

  // Description of this task.
  task.$desc = 'Set hosts'
  return task
}

module.exports = define
